<?php
/**
 * ownCloud
 *
 * @author Frank Karlitscheck <frank@owncloud.com>
 * @author Tom Needham <tom@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

// check if license app is enabled
if(!OCP\App::isEnabled('enterprise_key') && !getenv('CI')) {
	
	// enable accidently disabled license app. Can happen during upgrade.
	OCP\Util::writeLog('admin_audit','Enabling Enterprise license app which was deactivated.', \OCP\Util::ERROR);
	\OC::$server->getAppManager()->enableApp('enterprise_key');

} else {
	$eventDispatcher = OC::$server->getEventDispatcher();

	/**
	 * Log events of addition, deletion and update in the config.php file
	 * @var  $config
	 */
	$config = new OCA\Admin_Audit\Handlers\Config();
	$eventDispatcher->addListener('config.aftersetvalue', [$config, 'createorupdate']);
	$eventDispatcher->addListener('config.afterdeletevalue', [$config, 'delete']);

	/**
	 * Log events of addition, deletion and update in the appconfig table
	 * @var  $appConfig
	 */
	$appConfig = new OCA\Admin_Audit\Handlers\AppConfig();
	$eventDispatcher->addListener('appconfig.aftersetvalue', [$appConfig, 'createorupdate']);
	$eventDispatcher->addListener('appconfig.afterdeletevalue', [$appConfig, 'delete']);
	$eventDispatcher->addListener('appconfig.afterdeleteapp', [$appConfig, 'deleteapp']);

	/**
	 * Handle comment events to create, update and delete
	 * @var  $commentHandler
	 */
	$commentHandler = new OCA\Admin_Audit\Handlers\Comments();
	$eventDispatcher->addListener('comment.aftercreate', [$commentHandler, 'createComment']);
	$eventDispatcher->addListener('comment.afterupdate', [$commentHandler, 'updateComment']);
	$eventDispatcher->addListener('comment.afterdelete', [$commentHandler, 'deleteComment']);

	$sessionHandler = new OCA\Admin_Audit\Handlers\Session(new \OCP\Defaults());
	$eventDispatcher->addListener('user.beforelogin', [$sessionHandler, 'pre_login']);
	$eventDispatcher->addListener('user.afterlogin', [$sessionHandler, 'post_login']);
	$eventDispatcher->addListener('user.beforelogout', [$sessionHandler, 'logout']);


	$userSession = \OC::$server->getUserSession();
	$adminAuditUser = new OCA\Admin_Audit\Handlers\User();
	$eventDispatcher->addListener('user.afterfeaturechange', [$adminAuditUser, 'changeUser']);
	$eventDispatcher->addListener('user.aftersetpassword',[$adminAuditUser, 'postSetPassword']);
	$eventDispatcher->addListener('user.aftercreate', [$adminAuditUser, 'user_create']);
	$eventDispatcher->addListener('user.afterdelete', [$adminAuditUser, 'user_delete']);

	$groupManager = \OC::$server->getGroupManager();
	$groupManager->listen('\OC\Group', 'postAddUser',  ['\OCA\Admin_Audit\Handlers\User', 'postAddUser']);
	$groupManager->listen('\OC\Group', 'postRemoveUser',  ['\OCA\Admin_Audit\Handlers\User', 'postRemoveUser']);

	$fileAuditHandler = new OCA\Admin_Audit\Handlers\Files();
	$eventDispatcher->addListener('file.afterrename', [$fileAuditHandler, 'rename']);
	$eventDispatcher->addListener('file.aftercreate', [$fileAuditHandler, 'create']);
	$eventDispatcher->addListener('file.aftercopy', [$fileAuditHandler, 'copy']);
	$eventDispatcher->addListener('file.afterupdate', [$fileAuditHandler, 'update']);
	$eventDispatcher->addListener('file.beforedelete', [$fileAuditHandler, 'beforeDelete']);
	$eventDispatcher->addListener('file.afterdelete', [$fileAuditHandler, 'delete']);
	OCP\Util::connectHook(OC\Files\Filesystem::CLASSNAME, OC\Files\Filesystem::signal_read, '\OCA\Admin_Audit\Handlers\Files', 'read');

	OCP\Util::connectHook('\OCA\Files_Trashbin\Trashbin', 'post_restore', '\OCA\Admin_Audit\Handlers\Files', 'trash_post_restore');
	OCP\Util::connectHook('\OCP\Trashbin', 'preDelete', '\OCA\Admin_Audit\Handlers\Files', 'trash_delete');

	OCP\Util::connectHook('\OCP\Versions', 'delete', 'OCA\Admin_Audit\Handlers\Files', 'version_delete');
	OCP\Util::connectHook('\OCP\Versions', 'rollback', 'OCA\Admin_Audit\Handlers\Files', 'version_rollback');

	OCP\Util::connectHook('OCP\Share', 'post_shared', 'OCA\Admin_Audit\Handlers\Sharing', 'post_shared');
	OCP\Util::connectHook('OCP\Share', 'post_unshare', 'OCA\Admin_Audit\Handlers\Sharing', 'post_unshare');
	OCP\Util::connectHook('OCP\Share', 'share_link_access', 'OCA\Admin_Audit\Handlers\Sharing', 'share_link_access');

	//Listener for self unshare, update
	$sharing = new OCA\Admin_Audit\Handlers\Sharing();
	$eventDispatcher->addListener('fromself.unshare', [$sharing, 'unshareFromRecipient']);
	$eventDispatcher->addListener('share.afterupdate', [$sharing, 'shareUpdate']);

	//For logging CustomGroups activities
	$eventDispatcher = \OC::$server->getEventDispatcher();
	$customGroups = new OCA\Admin_Audit\Handlers\CustomGroups();
	$eventDispatcher->addListener('\OCA\CustomGroups::addGroupAndUser', [$customGroups,'addGroupAndUser']);
	$eventDispatcher->addListener('\OCA\CustomGroups::addUserToGroup', [$customGroups, 'addUserToGroup']);
	$eventDispatcher->addListener('\OCA\CustomGroups::deleteGroup', [$customGroups,'deleteGroup']);
	$eventDispatcher->addListener('\OCA\CustomGroups::removeUserFromGroup',[$customGroups,'removeUserFromGroup']);
	$eventDispatcher->addListener('\OCA\CustomGroups::leaveFromGroup',[$customGroups,'leaveFromGroup']);
	$eventDispatcher->addListener('\OCA\CustomGroups::updateGroupName', [$customGroups, 'updateGroupName']);
	$eventDispatcher->addListener('\OCA\CustomGroups::changeRoleInGroup', [$customGroups, 'changeRoleInGroup']);

	//For logging accepted/rejected federated share
	$eventDispatcher = OC::$server->getEventDispatcher();
	$acceptRemoteShare = new OCA\Admin_Audit\Handlers\Sharing();
	$eventDispatcher->addListener('remoteshare.accepted',[$acceptRemoteShare, 'accept_remote_shared']);
	$eventDispatcher->addListener('remoteshare.declined', [$acceptRemoteShare, 'decline_remote_shared']);
	$eventDispatcher->addListener('\OCA\FederatedFileSharing::remote_shareReceived', [$acceptRemoteShare, 'federatedShareArrivedFromRemote']);
	$eventDispatcher->addListener('\OCA\Files_Sharing::unshareEvent', [$acceptRemoteShare, 'unshareInfo']);

	$managerListener = function(\OCP\SystemTag\ManagerEvent $event) {
		$application = new \OCP\AppFramework\App('admin_audit');
		/** @var \OCA\Admin_Audit\Handlers\SystemTags $listener */
		$listener = $application->getContainer()->query('OCA\Admin_Audit\Handlers\SystemTags');
		$listener->managerEvent($event);
	};

	$eventDispatcher = \OC::$server->getEventDispatcher();
	$eventDispatcher->addListener(\OCP\SystemTag\ManagerEvent::EVENT_CREATE, $managerListener);
	$eventDispatcher->addListener(\OCP\SystemTag\ManagerEvent::EVENT_DELETE, $managerListener);
	$eventDispatcher->addListener(\OCP\SystemTag\ManagerEvent::EVENT_UPDATE, $managerListener);

	$mapperListener = function(\OCP\SystemTag\MapperEvent $event) {
		$application = new \OCP\AppFramework\App('admin_audit');
		/** @var \OCA\Admin_Audit\Handlers\SystemTags $listener */
		$listener = $application->getContainer()->query('OCA\Admin_Audit\Handlers\SystemTags');
		$listener->mapperEvent($event);
	};

	$eventDispatcher->addListener(\OCP\SystemTag\MapperEvent::EVENT_ASSIGN, $mapperListener);
	$eventDispatcher->addListener(\OCP\SystemTag\MapperEvent::EVENT_UNASSIGN, $mapperListener);

	$consoleListener = function(\OCP\Console\ConsoleEvent $event) {
		$application = new \OCP\AppFramework\App('admin_audit');
		/** @var \OCA\Admin_Audit\Handlers\Console $listener */
		$listener = $application->getContainer()->query('OCA\Admin_Audit\Handlers\Console');
		$listener->consoleEvent($event);
	};

	$eventDispatcher->addListener(\OCP\Console\ConsoleEvent::EVENT_RUN, $consoleListener);

	$appManagerListener = function(\OCP\App\ManagerEvent $event) {
		$application = new \OCP\AppFramework\App('admin_audit');
		/** @var \OCA\Admin_Audit\Handlers\App $listener */
		$listener = $application->getContainer()->query('OCA\Admin_Audit\Handlers\App');
		$listener->managerEvent($event);
	};

	$eventDispatcher->addListener(\OCP\App\ManagerEvent::EVENT_APP_ENABLE, $appManagerListener);
	$eventDispatcher->addListener(\OCP\App\ManagerEvent::EVENT_APP_ENABLE_FOR_GROUPS, $appManagerListener);
	$eventDispatcher->addListener(\OCP\App\ManagerEvent::EVENT_APP_DISABLE, $appManagerListener);

	$eventDispatcher->addListener( \OC\User\User::class . '::postSetEnabled', ['\OCA\Admin_Audit\Handlers\User', 'postSetEnabled']);

}
