<?php
/**
 * ownCloud Admin_Audit
 *
 * @author Sujith Haridasan <sharidasan@owncloud.com>
 * @copyright (C) 2017 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit\Handlers;


class CustomGroups extends Base {

	static public function leaveFromGroup($event) {
		self::logLeaveFromGroup('{actor} user {userId} leaves group {groupName}', $event);
	}

	static public function changeRoleInGroup($event) {
		self::logChangeRoleInGroup('{actor} user {user} changed role to {roleDisaplayName}, numeric value {roleNumber}, in group {groupName}', $event);
	}

	static public function updateGroupName($event) {
		self::logCustomGroupNameUpdate('{actor} group name {oldGroupName} is updated to {newGroupName}', $event);
	}

    static public function removeUserFromGroup($event) {
        self::logCustomGroupUserDelete('{actor}: {user} deleted from group {group}', $event);
    }

    static protected function logCustomGroupUserDelete($message, $arguments) {
        self::getLogger()->log($message,[
            'user' => $arguments['user_displayName'],
            'group' => $arguments['group_displayName'],
        ]);
    }

    static public function addGroupAndUser($event) {
        self::logCustomGroupCreated('{actor}: group {group} is newly created. User {user} is added to this group as admin', $event);
    }

    static protected function logLeaveFromGroup($message, $arguments) {
		self::getLogger()->log($message, [
			'userId' => $arguments['userId'],
			'groupName' => $arguments['groupName']
		]);
	}

    static protected function logChangeRoleInGroup($message, $arguments) {
		self::getLogger()->log($message, [
			'user' => $arguments['user'],
			'groupName' => $arguments['groupName'],
			'roleDisaplayName' => $arguments['roleDisaplayName'],
			'roleNumber' => $arguments['roleNumber']
		]);
	}

    static protected function logCustomGroupNameUpdate($message, $arguments) {
		self::getLogger()->log($message, [
			'oldGroupName' => $arguments['oldGroupName'],
			'newGroupName' => $arguments['newGroupName']
		]);
	}

    static protected function logCustomGroupCreated($message, $arguments) {
        self::getLogger()->log($message, [
            'group' => $arguments['groupName'],
            'user'  => $arguments['user']
        ]);
    }

    static public function deleteGroup($event) {
        self::logCustomGroupDeleted('{actor}: group {group} is deleted', $event);
    }

    static protected function logCustomGroupDeleted($message, $arguments) {
        self::getLogger()->log($message, [
            'group' => $arguments['groupName']
        ]);
    }

    static public function addUserToGroup($event) {
        self::logCustomGroupUserAdd('{actor}: user {user} is added to group {group}', $event);
    }

    static protected function logCustomGroupUserAdd($message, $arguments) {
        self::getLogger()->log($message, [
            'user' => $arguments['user'],
            'group' => $arguments['groupName']
        ]);
    }
}

